# Tweepy
# Copyright 2009 Joshua Roesslein
# See LICENSE

from . error import TweepError


class Model(object):

    def __getstate__(self):
        # pickle
        pickle = {}
        for k, v in self.__dict__.items():
            if k == '_api':
                # do not pickle the api reference
                continue
            pickle[k] = v
        return pickle

    @staticmethod
    def _validate(model, attributes):
        missing = []
        for attr in attributes:
            if not hasattr(model, attr):
                missing.append(attr)
        if len(missing) > 0:
            raise TweepError('Missing required attribute(s) %s' % \
                                str(missing).strip('[]'))

    def validate(self):
        return


class Status(Model):

    @staticmethod
    def _validate(status):
        Model._validate(status, [
          'created_at', 'id', 'text', 'source', 'truncated', 'in_reply_to_status_id',
          'in_reply_to_user_id', 'favorited', 'in_reply_to_screen_name'
        ])
        if hasattr(status, 'user'):
            User._validate(status.user)

    def validate(self):
        Status._validate(self)

    def destroy(self):
        return self._api.destroy_status(id=self.id)


class User(Model):

    @staticmethod
    def _validate(user):
        Model._validate(user, [
            'id', 'name', 'screen_name', 'location', 'description', 'profile_image_url',
            'url', 'protected', 'followers_count', 'profile_background_color',
            'profile_text_color', 'profile_sidebar_fill_color',
            'profile_sidebar_border_color', 'friends_count', 'created_at',
            'favourites_count', 'utc_offset', 'time_zone',
            'profile_background_image_url', 'statuses_count',
            'notifications', 'following', 'verified'
        ])
        if hasattr(user, 'status'):
            Status._validate(user.status)

    def validate(self):
        User._validate(self)

    def timeline(self, **kargs):
        return self._api.user_timeline(**kargs)

    def mentions(self, **kargs):
        return self._api.mentions(**kargs)

    def friends(self, **kargs):
        return self._api.friends(id=self.id, **kargs)

    def followers(self, **kargs):
        return self._api.followers(id=self.id, **kargs)

    def follow(self):
        self._api.create_friendship(user_id=self.id)
        self.following = True

    def unfollow(self):
        self._api.destroy_friendship(user_id=self.id)
        self.following = False


class DirectMessage(Model):

    def destroy(self):
        return self._api.destroy_direct_message(id=self.id)


class Friendship(Model):

    pass


class SavedSearch(Model):

    pass


class SearchResult(Model):

    pass

class Retweet(Model):

    pass

# link up default model implementations.
models = {
    'status': Status,
    'user': User,
    'direct_message': DirectMessage,
    'friendship': Friendship,
    'saved_search': SavedSearch,
    'search_result': SearchResult,
    'retweet': Retweet,
}




# Tweepy
# Copyright 2009 Joshua Roesslein
# See LICENSE

import httplib
from socket import timeout
from threading import Thread
from time import sleep

from . auth import BasicAuthHandler
from . parsers import parse_status
from . api import API
from . error import TweepError

try:
    import json #Python >= 2.6
except ImportError:
    try:
        import simplejson as json #Python < 2.6
    except ImportError:
        try:
            from django.utils import simplejson as json #Google App Engine
        except ImportError:
            raise ImportError, "Can't load a json library"

STREAM_VERSION = 1


class StreamListener(object):

    def on_status(self, status):
        """Called when a new status arrives"""
        return

    def on_delete(self, status_id, user_id):
        """Called when a delete notice arrives for a status"""
        return

    def on_limit(self, track):
        """Called when a limitation notice arrvies"""
        return

    def on_error(self, status_code):
        """Called when a non-200 status code is returned"""
        return False

    def on_timeout(self):
        """Called when stream connection times out"""
        return


class Stream(object):

    host = 'stream.twitter.com'

    def __init__(self, username, password, listener, timeout=5.0, retry_count = None,
                    retry_time = 10.0, snooze_time = 5.0, buffer_size=1500):
        self.auth = BasicAuthHandler(username, password)
        self.running = False
        self.timeout = timeout
        self.retry_count = retry_count
        self.retry_time = retry_time
        self.snooze_time = snooze_time
        self.buffer_size = buffer_size
        self.listener = listener
        self.api = API()

    def _run(self):
        # setup
        headers = {}
        self.auth.apply_auth(None, None, headers, None)

        # enter loop
        error_counter = 0
        conn = None
        while self.running:
            if self.retry_count and error_counter > self.retry_count:
                # quit if error count greater than retry count
                break
            try:
                conn = httplib.HTTPConnection(self.host)
                conn.connect()
                conn.sock.settimeout(self.timeout)
                conn.request('POST', self.url, headers=headers)
                resp = conn.getresponse()
                if resp.status != 200:
                    if self.listener.on_error(resp.status) is False:
                        break
                    error_counter += 1
                    sleep(self.retry_time)
                else:
                    error_counter = 0
                    self._read_loop(resp)
            except timeout:
                if self.listener.on_timeout() == False:
                    break
                if self.running is False:
                    break
                conn.close()
                sleep(self.snooze_time)
            except Exception:
                # any other exception is fatal, so kill loop
                break

        # cleanup
        self.running = False
        if conn:
            conn.close()

    def _read_loop(self, resp):
        data = ''
        while self.running:
            if resp.isclosed():
                break

            # read length
            length = ''
            while True:
                c = resp.read(1)
                if c == '\n':
                    break
                length += c
            length = length.strip()
            if length.isdigit():
                length = int(length)
            else:
                continue

            # read data
            data = resp.read(length)

            # turn json data into status object
            if 'in_reply_to_status_id' in data:
                status = parse_status(data, self.api)
                if self.listener.on_status(status) == False:
                    self.running = False
            elif 'delete' in data:
                delete = json.loads(data)['delete']['status']
                if self.listener.on_delete(delete['id'], delete['user_id']) == False:
                    self.running = False
            elif 'limit' in data:
                if self.listener.on_limit(json.loads(data)['limit']['track']) == False:
                    self.running = False

    def firehose(self, count=None):
        if self.running:
            raise TweepError('Stream object already connected!')
        self.url = '/%i/statuses/firehose.json?delimited=length' % STREAM_VERSION
        if count:
            self.url += '&count=%s' % count
        self.running = True
        Thread(target=self._run).start()

    def retweet(self):
        if self.running:
            raise TweepError('Stream object already connected!')
        self.url = '/%i/statuses/retweet.json?delimited=length' % STREAM_VERSION
        self.running = True
        Thread(target=self._run).start()

    def sample(self, count=None):
        if self.running:
            raise TweepError('Stream object already connected!')
        self.url = '/%i/statuses/sample.json?delimited=length' % STREAM_VERSION
        if count:
            self.url += '&count=%s' % count
        self.running = True
        Thread(target=self._run).start()

    def filter(self, follow=None, track=None):
        if self.running:
            raise TweepError('Stream object already connected!')
        self.url = '/%i/statuses/filter.json?delimited=length' % STREAM_VERSION
        if follow:
            self.url += '&follow=%s' % ','.join(follow)
        if track:
            self.url += '&track=%s' % ','.join(track)
        print self.url
        self.running = True
        Thread(target=self._run).start()

    def disconnect(self):
        if self.running is False:
            return
        self.running = False




# Tweepy
# Copyright 2009 Joshua Roesslein
# See LICENSE

"""
Tweepy Twitter API library
"""
__version__ = '1.1'

from . models import Status, User, DirectMessage, Friendship, SavedSearch, SearchResult, models
from . error import TweepError
from . api import API
from . cache import Cache, MemoryCache, FileCache, MemCache
from . auth import BasicAuthHandler, OAuthHandler
from . streaming import Stream, StreamListener
from . logging import TweepyLogger, DummyLogger, ConsoleLogger, FileLogger

# Global, unauthenticated instance of API
api = API()




# Tweepy
# Copyright 2009 Joshua Roesslein
# See LICENSE

from datetime import datetime

from . models import models

try:
    import json #Python >= 2.6
except ImportError:
    try:
        import simplejson as json #Python < 2.6
    except ImportError:
        try:
            from django.utils import simplejson as json #Google App Engine
        except ImportError:
            raise ImportError, "Can't load a json library"


def parse_json(data, api):

    return json.loads(data)


def parse_return_true(data, api):

    return True


def parse_none(data, api):

    return None


def parse_error(data):

    return json.loads(data)['error']


def _parse_datetime(str):

    return datetime.strptime(str, '%a %b %d %H:%M:%S +0000 %Y')


def _parse_search_datetime(str):

    return datetime.strptime(str, '%a, %d %b %Y %H:%M:%S +0000')


def _parse_html_value(html):

    return html[html.find('>')+1:html.rfind('<')]


def _parse_a_href(atag):

    return atag[atag.find('"')+1:atag.find('>')-1]


def _parse_user(obj, api):

    user = models['user']()
    user._api = api
    for k, v in obj.items():
        if k == 'created_at':
            setattr(user, k, _parse_datetime(v))
        elif k == 'status':
            setattr(user, k, _parse_status(v, api))
        elif k == 'following':
            # twitter sets this to null if it is false
            if v is True:
                setattr(user, k, True)
            else:
                setattr(user, k, False)
        else:
            setattr(user, k, v)
    return user


def parse_user(data, api):

    return _parse_user(json.loads(data), api)


def parse_users(data, api):

    users = []
    for obj in json.loads(data):
        users.append(_parse_user(obj, api))
    return users


def _parse_status(obj, api):

    status = models['status']()
    status._api = api
    for k, v in obj.items():
        if k == 'user':
            user = _parse_user(v, api)
            setattr(status, 'author', user)
            setattr(status, 'user', user)  # DEPRECIATED
        elif k == 'created_at':
            setattr(status, k, _parse_datetime(v))
        elif k == 'source':
            setattr(status, k, _parse_html_value(v))
            setattr(status, 'source_url', _parse_a_href(v))
        elif k == 'retweet_details':
            setattr(status, k, _parse_retweet(v, api))
        else:
            setattr(status, k, v)
    return status


def parse_status(data, api):

    return _parse_status(json.loads(data), api)


def parse_statuses(data, api):

    statuses = []
    for obj in json.loads(data):
        statuses.append(_parse_status(obj, api))
    return statuses


def _parse_dm(obj, api):

    dm = models['direct_message']()
    dm._api = api
    for k, v in obj.items():
        if k == 'sender' or k == 'recipient':
            setattr(dm, k, _parse_user(v, api))
        elif k == 'created_at':
            setattr(dm, k, _parse_datetime(v))
        else:
            setattr(dm, k, v)
    return dm


def parse_dm(data, api):

    return _parse_dm(json.loads(data), api)


def parse_directmessages(data, api):

    directmessages = []
    for obj in json.loads(data):
        directmessages.append(_parse_dm(obj, api))
    return directmessages


def parse_friendship(data, api):

    relationship = json.loads(data)['relationship']

    # parse source
    source = models['friendship']()
    for k, v in relationship['source'].items():
        setattr(source, k, v)

    # parse target
    target = models['friendship']()
    for k, v in relationship['target'].items():
        setattr(target, k, v)

    return source, target


def _parse_saved_search(obj, api):

    ss = models['saved_search']()
    ss._api = api
    for k, v in obj.items():
        if k == 'created_at':
            setattr(ss, k, _parse_datetime(v))
        else:
            setattr(ss, k, v)
    return ss


def parse_saved_search(data, api):

    return _parse_saved_search(json.loads(data), api)


def parse_saved_searches(data, api):

    saved_searches = []
    saved_search = models['saved_search']()
    for obj in json.loads(data):
        saved_searches.append(_parse_saved_search(obj, api))
    return saved_searches


def _parse_search_result(obj, api):

    result = models['search_result']()
    for k, v in obj.items():
        if k == 'created_at':
            setattr(result, k, _parse_search_datetime(v))
        else:
            setattr(result, k, v)
    return result


def parse_search_results(data, api):

    results = json.loads(data)['results']
    result_objects = []
    for obj in results:
        result_objects.append(_parse_search_result(obj, api))
    return result_objects

def _parse_retweet(obj, api):

    retweet = models['retweet']()
    for k,v in obj.items():
        if k == 'retweeting_user':
            setattr(retweet, k, _parse_user(v, api))
        else:
            setattr(retweet, k, v)
    return retweet

def parse_retweets(data, api):

    retweets = []
    for obj in json.loads(data):
        retweets.append(_parse_retweet(obj, api))
    return retweets




# Tweepy
# Copyright 2009 Joshua Roesslein
# See LICENSE

import httplib
import urllib

from . parsers import parse_error
from . error import TweepError


def bind_api(path, parser, allowed_param=None, method='GET', require_auth=False,
              timeout=None, host=None):

    def _call(api, *args, **kargs):
        # If require auth, throw exception if credentials not provided
        if require_auth and not api.auth_handler:
            raise TweepError('Authentication required!')

        # Log some useful infomation
        api.logger.debug('Starting request...')
        api.logger.debug('  path: %s' % path)
        api.logger.debug('  method: %s' % method)

        # check for post_data parameter
        if 'post_data' in kargs:
            post_data = kargs['post_data']
            del kargs['post_data']
            api.logger.debug('  post data: %s' % post_data)
        else:
            post_data = None

        # check for headers
        if 'headers' in kargs:
            headers = dict(kargs['headers'])
            del kargs['headers']
        else:
            headers = {}
        api.logger.debug('  headers: %s' % headers)

        # build parameter dict
        if allowed_param:
            parameters = {}
            for idx, arg in enumerate(args):
                try:
                    parameters[allowed_param[idx]] = arg
                except IndexError:
                    raise TweepError('Too many parameters supplied!')
            for k, arg in kargs.items():
                if arg is None:
                    continue
                if k in parameters:
                    raise TweepError('Multiple values for parameter %s supplied!' % k)
                if k not in allowed_param:
                    raise TweepError('Invalid parameter %s supplied!' % k)
                parameters[k] = arg
        else:
            if len(args) > 0 or len(kargs) > 0:
                raise TweepError('This method takes no parameters!')
            parameters = None
        api.logger.debug('  parameters: %s' % parameters)

        # Build url with parameters
        if parameters:
            url = '%s?%s' % (api.api_root + path, urllib.urlencode(parameters))
        else:
            url = api.api_root + path

        # get scheme and host
        if api.secure:
            scheme = 'https://'
        else:
            scheme = 'http://'
        _host = host or api.host

        # Apply authentication
        if api.auth_handler:
            api.auth_handler.apply_auth(
                    scheme + _host + url,
                    method, headers, parameters
            )

        # Check cache if caching enabled and method is GET
        if api.cache and method == 'GET':
            cache_result = api.cache.get(url, timeout)
            # if cache result found and not expired, return it
            if cache_result:
                # must restore api reference
                if isinstance(cache_result, list):
                    for result in cache_result:
                        result._api = api
                else:
                    cache_result._api = api
                api.logger.debug("Cache hit!")
                return cache_result

        # Open connection
        # FIXME: add timeout
        if api.secure:
            conn = httplib.HTTPSConnection(_host)
        else:
            conn = httplib.HTTPConnection(_host)

        # Build request
        conn.request(method, url, headers=headers, body=post_data)

        # Get response
        resp = conn.getresponse()
        api.last_response = resp
        api.logger.debug('Received response...')
        api.logger.debug('  headers: %s' % resp.getheaders())
        api.logger.debug('  status code: %s' % resp.status)

        # If an error was returned, throw an exception
        if resp.status != 200:
            try:
                error_msg = parse_error(resp.read())
            except Exception:
                error_msg = "Twitter error response: status code = %s" % resp.status
            api.logger.error('  Error: %s' % error_msg)
            raise TweepError(error_msg)

        # Pass returned body into parser and return parser output
        try:
            out = parser(resp.read(), api)
        except Exception:
            api.logger.error("  parse error!")
            raise TweepError("Failed to parse returned data")

        conn.close()

        # validate result
        if api.validate:
            # list of results
            if isinstance(out, list) and len(out) > 0:
                if hasattr(out[0], 'validate'):
                    for result in out:
                        result.validate()
            # single result
            else:
                if hasattr(out, 'validate'):
                    out.validate()

        # store result in cache
        if api.cache and method == 'GET':
            api.cache.store(url, out)
            api.logger.debug("  caching result")

        api.logger.debug('request done.')

        return out

    return _call




# Tweepy
# Copyright 2009 Joshua Roesslein
# See LICENSE

import os
import mimetypes

from . binder import bind_api
from . error import TweepError
from . auth import BasicAuthHandler, OAuthHandler
from . logging import DummyLogger
from tweepy.parsers import *


class API(object):
    """Twitter API"""

    def __init__(self, auth_handler=None, host='twitter.com', cache=None,
            secure=False, api_root='', validate=True, logger=DummyLogger()):
        # you may access these freely
        self.auth_handler = auth_handler
        self.host = host
        self.api_root = api_root
        self.cache = cache
        self.secure = secure
        self.validate = validate
        self.logger = logger

        # not a good idea to touch these
        self._username = None

    @staticmethod
    def new(auth='basic', *args, **kargs):
        if auth == 'basic':
            return API(BasicAuthHandler(*args, **kargs))
        elif auth == 'oauth':
            return API(OAuthHandler(*args, **kargs))
        else:
            raise TweepError('Invalid auth type')

    """ statuses/public_timeline

        Returns the 20 most recent statuses from non-protected users who
        have set a custom user icon. The public timeline is cached for 60 
        seconds so requesting it more often than that is a waste of resources.

        Parameters: None
        Returns: list[Status]

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-statuses-public_timeline
	"""
    public_timeline = bind_api(
        path = '/statuses/public_timeline.json',
        parser = parse_statuses,
        allowed_param = []
    )

    """ statuses/home_timeline [Coming soon]

        Returns the 20 most recent statuses, including retweets, posted
        by the authenticating user and that user's friends. This is the
        equivalent of /timeline/home on the Web.

        Parameters: since_id, max_id, count, page
        Returns: list[Status]

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-statuses-home_timeline
    """
    home_timeline = bind_api(
        path = '/statuses/home_timeline.json',
        parser = parse_statuses,
        allowed_param = ['since_id', 'max_id', 'count', 'page'],
        require_auth = True
    )

    """ statuses/friends_timeline

        Returns the 20 most recent statuses posted by the
        authenticating user and that user's friends.

        Parameters: since_id, max_id, count, page
        Returns: list[Status]

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-statuses-friends_timeline
    """
    friends_timeline = bind_api(
        path = '/statuses/friends_timeline.json',
        parser = parse_statuses,
        allowed_param = ['since_id', 'max_id', 'count', 'page'],
        require_auth = True
    )

    """ statuses/user_timeline

        Returns the 20 most recent statuses posted from the
        authenticating user. It's also possible to request another
        user's timeline via the id parameter.

        Parameters: (id or user_id or screen_name), since_id, max_id, count, page
        Returns: list[Status]

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-statuses-user_timeline
    """
    user_timeline = bind_api(
        path = '/statuses/user_timeline.json',
        parser = parse_statuses,
        allowed_param = ['id', 'user_id', 'screen_name', 'since_id',
                          'max_id', 'count', 'page']
    )

    """ statuses/mentions

        Returns the 20 most recent mentions (status containing @username)
        for the authenticating user.

        Parameters: since_id, max_id, count, page
        Returns: list[Status]

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-statuses-mentions
    """
    mentions = bind_api(
        path = '/statuses/mentions.json',
        parser = parse_statuses,
        allowed_param = ['since_id', 'max_id', 'count', 'page'],
        require_auth = True
    )

    """ statuses/retweeted_by_me [Coming soon]

        Returns the 20 most recent retweets posted by the authenticating user.

        Parameters: since_id, max_id, count, page
        Returns: list[Status]

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-statuses-retweeted_by_me
    """
    retweeted_by_me = bind_api(
        path = '/statuses/retweeted_by_me.json',
        parser = parse_statuses,
        allowed_param = ['since_id', 'max_id', 'count', 'page'],
        require_auth = True
    )

    """ statuses/retweeted_to_me

        Returns the 20 most recent retweets posted by the
        authenticating user's friends.

        Parameters: since_id, max_id, count, page
        Returns: list[Status]

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-statuses-retweeted_to_me
    """
    retweeted_to_me = bind_api(
        path = '/statuses/retweeted_to_me.json',
        parser = parse_statuses,
        allowed_param = ['since_id', 'max_id', 'count', 'page'],
        require_auth = True
    )

    """ statuses/retweets_of_me

        Returns the 20 most recent tweets of the authenticated
        user that have been retweeted by others.

        Parameters: since_id, max_id, count, page
        Returns: list[Status]

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-statuses-retweets_of_me
    """
    retweets_of_me = bind_api(
        path = '/statuses/retweets_of_me.json',
        parser = parse_statuses,
        allowed_param = ['since_id', 'max_id', 'count', 'page'],
        require_auth = True
    )

    """ statuses/show

        Returns a single status, specified by the id parameter below.
        The status's author will be returned inline.

        Parameters: id (Required)
        Returns: Status

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-statuses%C2%A0show
    """
    get_status = bind_api(
        path = '/statuses/show.json',
        parser = parse_status,
        allowed_param = ['id']
    )

    """ statuses/update [Geolocation parameters coming soon]

        Updates the authenticating user's status.
        A status update with text identical to the authenticating user's
        current status will be ignored to prevent duplicates.

        Parameters: status (Required), in_reply_to_status_id, lat, long
        Returns: Status

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-statuses%C2%A0update
    """
    update_status = bind_api(
        path = '/statuses/update.json',
        method = 'POST',
        parser = parse_status,
        allowed_param = ['status', 'in_reply_to_status_id', 'lat', 'long'],
        require_auth = True
    )

    """ statuses/destroy

        Destroys the status specified by the required ID parameter.
        The authenticating user must be the author of the specified status.

        Parameters: id (Required)
        Returns: Status
    """
    destroy_status = bind_api(
        path = '/statuses/destroy.json',
        method = 'DELETE',
        parser = parse_status,
        allowed_param = ['id'],
        require_auth = True
    )

    """ statuses/retweet [Coming soon]

        Retweets a tweet. Requires the id parameter of the tweet you are
        retweeting. Returns the original tweet with retweet details.

        Parameters: id (Required)
        Returns: Status

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-statuses-retweet
    """
    retweet = bind_api(
        path = '/statuses/retweet/id.json',
        method = 'POST',
        parser = parse_status,
        allowed_param = ['id'],
        require_auth = True
    )

    """ statuses/retweets [Coming soon]

        Returns up to 100 of the first retweets of a given tweet.

        Parameters: id (Required), count
        Returns: Retweet

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-statuses-retweets
    """
    def retweets(self, id, *args, **kargs):
        return bind_api(
            path = '/statuses/retweets/%s.json' % id,
            parser = parse_retweets,
            allowed_param = ['count'],
            require_auth = True
        )(self, *args, **kargs)

    """ users/show

        Returns extended information of a given user, specified by ID or
        screen name as per the required id parameter.

        Parameters: id or user_id or screen_name (One of these is required)
        Returns: User

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-users%C2%A0show
    """
    get_user = bind_api(
        path = '/users/show.json',
        parser = parse_user,
        allowed_param = ['id', 'user_id', 'screen_name']
    )

    """ Get the authenticated user

        Equivalent of calling API.get_user(authenticated_user_name)

        Parameters: None
        Returns: User

        See: API.get_user()
    """
    def me(self):
        # if username not fetched, go get it...
        if self._username is None:
            if self.auth_handler is None:
                raise TweepError('Authentication required')

            try:
                user = bind_api(
                    path = '/account/verify_credentials.json',
                    parser = parse_user
                )(self)
            except TweepError, e:
                raise TweepError('Failed to fetch username: %s' % e)

            self._username = user.screen_name

        return self.get_user(screen_name=self._username)

    """ statuses/friends

        Returns a user's friends. They are ordered by the order in
        which they were added as friends, 100 at a time.
        (Please note that the result set isn't guaranteed to be 100
        every time as suspended users will be filtered out.)
        Use the page option to access older friends. With no user specified,
        request defaults to the authenticated user's friends. It's also
        possible to request another user's friends list via the id,
        screen_name or user_id parameter.

        Parameters: (id or user_id or screen_name), page
        Returns: list[Users]

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-statuses%C2%A0friends
    """
    friends = bind_api(
        path = '/statuses/friends.json',
        parser = parse_users,
        allowed_param = ['id', 'user_id', 'screen_name', 'page']
    )

    """ statuses/followers

        Returns the authenticating user's followers.They are ordered by
        the order in which they joined Twitter, 100 at a time.
        (Please note that the result set isn't guaranteed to be 100 every
        time as suspended users will be filtered out.)
        Use the page option to access earlier followers.

        Parameters: (id or user_id or screen_name), page
        Returns: list[User]

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-statuses%C2%A0followers
    """
    followers = bind_api(
        path = '/statuses/followers.json',
        parser = parse_users,
        allowed_param = ['id', 'user_id', 'screen_name', 'page'],
        require_auth = True
    )

    """ direct_messages

        Returns a list of the 20 most recent direct messages sent
        to the authenticating user.

        Parameters: since_id, max_id, count, page
        Returns: list[DirectMessage]

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-direct_messages
    """
    direct_messages = bind_api(
        path = '/direct_messages.json',
        parser = parse_directmessages,
        allowed_param = ['since_id', 'max_id', 'count', 'page'],
        require_auth = True
    )

    """ direct_messages/sent

        Returns a list of the 20 most recent direct messages sent
        by the authenticating user.

        Parameters: since_id, max_id, count, page
        Returns: list[DirectMessage]

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-direct_messages%C2%A0sent
    """
    sent_direct_messages = bind_api(
        path = '/direct_messages/sent.json',
        parser = parse_directmessages,
        allowed_param = ['since_id', 'max_id', 'count', 'page'],
        require_auth = True
    )

    """ direct_messages/new

        Sends a new direct message to the specified user from
        the authenticating user.

        Parameters: user (Required), text (Required)
        Returns: DirectMessage

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-direct_messages%C2%A0new
    """
    send_direct_message = bind_api(
        path = '/direct_messages/new.json',
        method = 'POST',
        parser = parse_dm,
        allowed_param = ['user', 'text'],
        require_auth = True
    )

    """ direct_messages/destroy

        Destroys the direct message specified in the required ID parameter.
        The authenticating user must be the recipient of the
        specified direct message.

        Parameters: id (Required)
        Returns: DirectMessage

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-direct_messages%C2%A0destroy
    """
    destroy_direct_message = bind_api(
        path = '/direct_messages/destroy.json',
        method = 'DELETE',
        parser = parse_dm,
        allowed_param = ['id'],
        require_auth = True
    )

    """ friendships/create

        Allows the authenticating users to follow the user specified in
        the ID parameter.  Returns the befriended user when successful.

        Parameters:
            id or user_id or screen_name (One of these is required)
            follow
        Returns: User

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-friendships%C2%A0create
    """
    create_friendship = bind_api(
        path = '/friendships/create.json',
        method = 'POST',
        parser = parse_user,
        allowed_param = ['id', 'user_id', 'screen_name', 'follow'],
        require_auth = True
    )

    """ friendships/destroy

        Allows the authenticating users to unfollow the user specified
        in the ID parameter.  Returns the unfollowed user when successful.

        Parameters: id or user_id or screen_name (One of these is required)
        Returns: User

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-friendships%C2%A0destroy
    """
    destroy_friendship = bind_api(
        path = '/friendships/destroy.json',
        method = 'DELETE',
        parser = parse_user,
        allowed_param = ['id', 'user_id', 'screen_name'],
        require_auth = True
    )

    """ friendships/exists

        Tests for the existence of friendship between two users.

        Parameters: user_a (Required), user_b (Required)
        Returns: Boolean (True if user_a follows user_b, otherwise False)

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-friendships-exists
    """
    exists_friendship = bind_api(
        path = '/friendships/exists.json',
        parser = parse_json,
        allowed_param = ['user_a', 'user_b']
    )

    """ friendships/show

        Returns detailed information about the relationship between two users.

        Parameters:
            source_id or source_screen_name (One of these is required)
            target_id or target_screen_name (One of these is required)
        Returns: tuple(Friendship - source, Friendship - target)

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-friendships-show
    """
    show_friendship = bind_api(
        path = '/friendships/show.json',
        parser = parse_friendship,
        allowed_param = ['source_id', 'source_screen_name',
                          'target_id', 'target_screen_name']
    )

    """ friends/ids

        Returns an array of numeric IDs for every user the
        specified user is following.

        Parameters:
            id or user_id or screen_name (One of these is required)
            cursor
        Returns: json object

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-friends%C2%A0ids
    """
    friends_ids = bind_api(
        path = '/friends/ids.json',
        parser = parse_json,
        allowed_param = ['id', 'user_id', 'screen_name', 'cursor']
    )

    """ followers/ids

        Returns an array of numeric IDs for every user following
        the specified user.

        Parameters:
            id or user_id or screen_name (One of these is required)
            cursor
        Returns: json object

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-followers%C2%A0ids
    """
    followers_ids = bind_api(
        path = '/followers/ids.json',
        parser = parse_json,
        allowed_param = ['id', 'user_id', 'screen_name', 'cursor']
    )

    """ account/verify_credentials

        Use this method to test if supplied user credentials are valid.
        Because this method can be a vector for a brute force dictionary
        attack to determine a user's password, it is limited to 15 requests
        per 60 minute period (starting from your first request).

        Parameters: None
        Returns: True if credentials are valid, otherwise False

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-account%C2%A0verify_credentials
    """
    def verify_credentials(self):
        try:
            return bind_api(
                path = '/account/verify_credentials.json',
                parser = parse_return_true,
                require_auth = True
            )(self)
        except TweepError:
            return False

    """ account/rate_limit_status

        Returns the remaining number of API requests available to the
        requesting user before the API limit is reached for the current hour.
        Calls to rate_limit_status do not count against the rate limit.
        If authentication credentials are provided, the rate limit status for
        the authenticating user is returned.  Otherwise, the rate limit status
        for the requester's IP address is returned.

        Parameters: None
        Returns: json object

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-account%C2%A0rate_limit_status
    """
    rate_limit_status = bind_api(
        path = '/account/rate_limit_status.json',
        parser = parse_json
    )

    """ account/update_delivery_device

        Sets which device Twitter delivers updates to for the authenticating
        user.  Sending "none" as the device parameter will disable IM or SMS
        updates.

        Parameters: device (Required)
        Returns: User

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-account%C2%A0update_delivery_device
    """
    set_delivery_device = bind_api(
        path = '/account/update_delivery_device.json',
        method = 'POST',
        allowed_param = ['device'],
        parser = parse_user,
        require_auth = True
    )

    """ account/update_profile_colors

        Sets one or more hex values that control the color scheme of the
        authenticating user's profile page on twitter.com.

        Parameters: profile_background_color, profile_text_color,
            profile_link_color, profile_sidebar_fill_color,
            profile_sidebar_border_color
        Returns: User

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-account%C2%A0update_profile_colors
    """
    update_profile_colors = bind_api(
        path = '/account/update_profile_colors.json',
        method = 'POST',
        parser = parse_user,
        allowed_param = ['profile_background_color', 'profile_text_color',
                          'profile_link_color', 'profile_sidebar_fill_color',
                          'profile_sidebar_border_color'],
        require_auth = True
    )

    """ account/update_profile_image

        Updates the authenticating user's profile image.

        Parameters:
            filename - local file path to image (Required)
        Returns: User

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-account%C2%A0update_profile_image
    """
    def update_profile_image(self, filename):
        headers, post_data = _pack_image(filename, 700)
        bind_api(
            path = '/account/update_profile_image.json',
            method = 'POST',
            parser = parse_none,
            require_auth = True
        )(self, post_data=post_data, headers=headers)

    """ account/update_profile_background_image

        Updates the authenticating user's profile background image.

        Parameters:
            filename - local file path to image (Required)
            tile
        Returns: User

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-account%C2%A0update_profile_background_image
    """
    def update_profile_background_image(self, filename, *args, **kargs):
        headers, post_data = _pack_image(filename, 800)
        bind_api(
            path = '/account/update_profile_background_image.json',
            method = 'POST',
            parser = parse_none,
            allowed_param = ['tile'],
            require_auth = True
        )(self, post_data=post_data, headers=headers)

    """ account/update_profile

        Sets values that users are able to set under the "Account" tab of
        their settings page. Only the parameters specified will be updated.

        Parameters: name, email, url, location, description
        Returns: User

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-account%C2%A0update_profile
    """
    update_profile = bind_api(
        path = '/account/update_profile.json',
        method = 'POST',
        parser = parse_user,
        allowed_param = ['name', 'email', 'url', 'location', 'description'],
        require_auth = True
    )

    """ favorites

        Returns the 20 most recent favorite statuses for the authenticating
        user or user specified by the ID parameter.

        Parameters: id, page
        Returns: list[Status]

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-favorites
    """
    favorites = bind_api(
        path = '/favorites.json',
        parser = parse_statuses,
        allowed_param = ['id', 'page']
    )

    """ favorites/create

        Favorites the status specified in the ID parameter as the
        authenticating user. Returns the favorite status when successful.

        Parameters: id (Required)
        Returns: Status

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-favorites%C2%A0create
    """
    create_favorite = bind_api(
        path = '/favorites/create.json',
        method = 'POST',
        parser = parse_status,
        allowed_param = ['id'],
        require_auth = True
    )

    """ favorites/destroy

        Un-favorites the status specified in the ID parameter as the
        authenticating user. Returns the un-favorited status.

        Parameters: id (Required)
        Returns: Status

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-favorites%C2%A0destroy
    """
    destroy_favorite = bind_api(
        path = '/favorites/destroy.json',
        method = 'DELETE',
        parser = parse_status,
        allowed_param = ['id'],
        require_auth = True
    )

    """ notifications/follow

        Enables device notifications for updates from the specified user.
        Returns the specified user when successful.

        Parameters: id or user_id or screen_name (One of these is required)
        Returns: User

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-notifications%C2%A0follow
    """
    enable_notifications = bind_api(
        path = '/notifications/follow.json',
        method = 'POST',
        parser = parse_user,
        allowed_param = ['id', 'user_id', 'screen_name'],
        require_auth = True
    )

    """ notifications/leave

        Disables notifications for updates from the specified user to the
        authenticating user.  Returns the specified user when successful.

        Parameters: id or user_id or screen_name (One of these is required)
        Returns: User

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-notifications%C2%A0leave
    """
    disable_notifications = bind_api(
        path = '/notifications/leave.json',
        method = 'POST',
        parser = parse_user,
        allowed_param = ['id', 'user_id', 'screen_name'],
        require_auth = True
    )

    """ blocks/create

        Blocks the user specified in the ID parameter as the authenticating
        user. Destroys a friendship to the blocked user if it exists.
        Returns the blocked user when successful.

        Parameters: id (Required)
        Returns: User

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-blocks%C2%A0create
    """
    create_block = bind_api(
        path = '/blocks/create.json',
        method = 'POST',
        parser = parse_user,
        allowed_param = ['id'],
        require_auth = True
    )

    """ blocks/destroy

        Un-blocks the user specified in the ID parameter for the
        authenticating user. Returns the un-blocked user when successful.

        Parameters: id (Required)
        Returns: User

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-blocks%C2%A0destroy
    """
    destroy_block = bind_api(
        path = '/blocks/destroy.json',
        method = 'DELETE',
        parser = parse_user,
        allowed_param = ['id'],
        require_auth = True
    )

    """ blocks/exists

        Checks if the authenticating user is blocking a target user.

        Parameters: id or user_id or screen_name (One of these is required)
        Returns: Boolean (True if blocked, otherwise False)

        http://apiwiki.twitter.com/Twitter+REST+API+Method%3A-blocks-exists
    """
    def exists_block(self, **kargs):
        try:
            bind_api(
                path = '/blocks/exists.json',
                parser = parse_none,
                allowed_param = ['id', 'user_id', 'screen_name'],
                require_auth = True
            )(self, **kargs)
        except TweepError:
            return False

        return True

    """ blocks/blocking

        Returns an array of user objects that the authenticating
        user is blocking.

        Parameters: page
        Returns: list[User]

        http://apiwiki.twitter.com/Twitter+REST+API+Method%3A-blocks-blocking
    """
    blocks = bind_api(
        path = '/blocks/blocking.json',
        parser = parse_users,
        allowed_param = ['page'],
        require_auth = True
    )

    """ blocks/blocking/ids

        Returns an array of numeric user ids the authenticating
        user is blocking.

        Parameters: None
        Returns: json object

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-blocks-blocking-ids
    """
    blocks_ids = bind_api(
        path = '/blocks/blocking/ids.json',
        parser = parse_json,
        require_auth = True
    )

    """ saved_searches

        Returns the authenticated user's saved search queries.

        Parameters: None
        Returns: list[SavedSearch]

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-saved_searches
    """
    saved_searches = bind_api(
        path = '/saved_searches.json',
        parser = parse_saved_searches,
        require_auth = True
    )

    """ saved_searches/show

        Retrieve the data for a saved search owned by the
        authenticating user specified by the given id.

        Parameters: id (Required)
        Returns: SavedSearch

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-saved_searches-show
    """
    def get_saved_search(self, id):
        return bind_api(
            path = '/saved_searches/show/%s.json' % id,
            parser = parse_saved_search,
            require_auth = True
        )(self)

    """ saved_searches/create

        Creates a saved search for the authenticated user.

        Parameters: query (Required)
        Returns: SavedSearch

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-saved_searches-create
    """
    create_saved_search = bind_api(
        path = '/saved_searches/create.json',
        method = 'POST',
        parser = parse_saved_search,
        allowed_param = ['query'],
        require_auth = True
    )

    """ saved_searches/destroy

        Destroys a saved search for the authenticated user.
        The search specified by id must be owned by the authenticating user.

        Parameters: id (Required)
        Returns: SavedSearch

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-saved_searches-destroy
    """
    def destroy_saved_search(self, id):
        return bind_api(
            path = '/saved_searches/destroy/%s.json' % id,
            method = 'DELETE',
            parser = parse_saved_search,
            allowed_param = ['id'],
            require_auth = True
        )(self)

    """ help/test

        Invokes the test method in the Twitter API.

        Parameters: None
        Returns: Boolean (True if 200 status code returned, otherwise False)

        http://apiwiki.twitter.com/Twitter-REST-API-Method%3A-help%C2%A0test
    """
    def test(self):
        try:
            return bind_api(
                path = '/help/test.json',
                parser = parse_return_true
            )(self)
        except TweepError:
            return False

    """ search

        Returns tweets that match a specified query.

        Parameters: q (Required), lang, locale, rpp, page, since_id
            geocode, show_user
        Returns: list[SearchResult]

        http://apiwiki.twitter.com/Twitter-Search-API-Method%3A-search
    """

    def search(self, *args, **kargs):
        return bind_api(
            host = 'search.' + self.host,
            path = '/search.json',
            parser = parse_search_results,
            allowed_param = ['q', 'lang', 'locale', 'rpp', 'page', 'since_id', 'geocode', 'show_user'],
        )(self, *args, **kargs)

    """ trends

        Returns the top ten topics that are currently trending on Twitter.
        The response includes the time of the request, the name of each trend,
        and the url to the Twitter Search results page for that topic.

        Parameters: None
        Returns: json object

        http://apiwiki.twitter.com/Twitter-Search-API-Method%3A-trends
    """
    def trends(self):
        return bind_api(
            host = 'search.' + self.host,
            path = '/trends.json',
            parser = parse_json
        )(self)

    """ trends/current

        Returns the current top 10 trending topics on Twitter.
        The response includes the time of the request, the name of each
        trending topic, and query used on Twitter Search results
        page for that topic.

        Parameters: exclude
        Returns: json object

        http://apiwiki.twitter.com/Twitter-Search-API-Method%3A-trends-current
    """
    def trends_current(self, *args, **kargs):
        return bind_api(
            host = 'search.' + self.host,
            path = '/trends/current.json',
            parser = parse_json,
            allowed_param = ['exclude']
        )(self, *args, **kargs)

    """ trends/daily

        Returns the top 20 trending topics for each hour in a given day.

        Parameters: date, exclude
        Returns: json object

        http://apiwiki.twitter.com/Twitter-Search-API-Method%3A-trends-daily
    """
    def trends_daily(self, *args, **kargs):
        return bind_api(
            host = "search." + self.host,
            path = '/trends/daily.json',
            parser = parse_json,
            allowed_param = ['date', 'exclude']
        )(self, *args, **kargs)

    """ trends/weekly

        Returns the top 30 trending topics for each day in a given week.

        Parameters: date, exclude
        Returns: json object

        http://apiwiki.twitter.com/Twitter-Search-API-Method%3A-trends-weekly
    """
    def trends_weekly(self, *args, **kargs):
        return bind_api(
            host = "search." + self.host,
            path = '/trends/weekly.json',
            parser = parse_json,
            allowed_param = ['date', 'exclude']
        )(self, *args, **kargs)

    """ Internal use only """

    def _pack_image(filename, max_size):
        """Pack image from file into multipart-formdata post body"""
        # image must be less than 700kb in size
        try:
            if os.path.getsize(filename) > (max_size * 1024):
                raise TweepError('File is too big, must be less than 700kb.')
        except os.error, e:
            raise TweepError('Unable to access file')

        # image must be gif, jpeg, or png
        file_type = mimetypes.guess_type(filename)
        if file_type is None:
            raise TweepError('Could not determine file type')
        file_type = file_type[0]
        if file_type not in ['image/gif', 'image/jpeg', 'image/png']:
            raise TweepError('Invalid file type for image: %s' % file_type)

        # build the mulitpart-formdata body
        fp = open(filename, 'rb')
        BOUNDARY = 'Tw3ePy'
        body = []
        body.append('--' + BOUNDARY)
        body.append('Content-Disposition: form-data; name="image"; filename="%s"' % filename)
        body.append('Content-Type: %s' % file_type)
        body.append('')
        body.append(fp.read())
        body.append('--' + BOUNDARY + '--')
        body.append('')
        fp.close()
        body = '\r\n'.join(body)

        # build headers
        headers = {
            'Content-Type': 'multipart/form-data; boundary=Tw3ePy',
            'Content-Length': len(body)
        }

        return headers, body




# Tweepy
# Copyright 2009 Joshua Roesslein
# See LICENSE

class TweepyLogger(object):

    DEBUG = 1
    WARNING = 2
    ERROR = 3

    def debug(self, message):
        """Output a debug log message"""
        self.log(TweepyLogger.DEBUG, message)

    def warning(self, message):
        """Output warning log message"""
        self.log(TweepyLogger.WARNING, message)

    def error(self, message):
        """Output error log message"""
        self.log(TweepyLogger.ERROR, message)

    def log(self, level, message):
        """Implement this method to handle log messages"""
        raise NotImplementedError

    def format(self, message):
        """Override this method to apply custom formating of messages"""
        return message

class DummyLogger(TweepyLogger):
    """This logger just discards log messages"""

    def log(self, level, message):
        return

class ConsoleLogger(TweepyLogger):
    """Outputs log messages to stdout"""

    def __init__(self, active_log_level=TweepyLogger.DEBUG):
        self.active_log_level = active_log_level

    def log(self, level, message):
        if level <= self.active_log_level:
            print message

class FileLogger(TweepyLogger):
    """Outputs log message to file"""

    def __init__(self, filepath, active_log_level=TweepyLogger.DEBUG):
        self.active_log_level = active_log_level
        self.file = open(filepath, 'w')

    def log(self, level, message):
        if level <= self.active_log_level:
            self.file.write(message + '\n')
            self.file.flush()

